#include <GC.h>
#include "shared.h"

extern uint8 cart_file[ ];

int ingame = 0;
int paused = 0;
GC_Pad gc_Pad;

unsigned char *keystate;
unsigned long *FrameBuffer;
#define PALSIZE ((1<<15)-1)
unsigned long palette[PALSIZE];
void SetPalette();

#define TIME_ONE_MS 40000
#define TIME_SEC(s) ((s)*TIME_ONE_MS*1000)
#define TIME_FRAME 666666

//Update controller input
void update_input();

int main (int argc, char **argv)
{
	u16 genvideo[307200];
	u16 cenx, ceny;
	u16 lastw=0, lasth=0;
	u32 vidw, vidh;
	u32 vidx, vidy;
	u16 *mpointer;
	u8 audiostarted = 0;
	unsigned long tempcolor,pos,pos2;

	GC_Time lasttime=0;
	//GC_Time offby=0;
	u16 framecheck = 0;
	//Every second the auto frameskip will be behind 40 ticks.
	//This is used to sync it back with Vsync


	/* Initialize GC stuff */
	// Initialize the system.
	GC_System_Init ();
	// Video and framebuffer initialization.
	FrameBuffer = (unsigned long*)0xC1000000;
	// Set NTSC mode.
	GC_Video_Init (VI_NTSC_640x480);
	GC_Debug_Init (FrameBuffer, VI_NTSC_640x480);
	// Set the framebuffer.
	//GC_Video_SetFrameBuffer ((void*)0x10080000, VI_FRAMEBUFFER_BOTH);
	*(unsigned long*)0xCC00201C = 0x10080000;
	*(unsigned long*)0xCC002024 = 0x10080000;
	// Clear framebuffer to black.
	GC_Video_ClearFrameBuffer (FrameBuffer, 0x00800080);
	// Wait two frames to allow TV to sync.
	GC_Video_WaitForVBlank ();
	GC_Video_WaitForVBlank ();
	// Enable interrupts.
	GC_Interrupts_Enable ();
	GC_Memory_InitHeap((void*)0x81100000,(void*)0x81500000);
	/* Done with GC stuff */

	
	GC_Debug_RStringT ("SMS Plus Gamecube port BETA 0.1", 0xFF80FF80, 40, 60);

	GC_Debug_RStringT ("This emulator is free, and open source.", 0xFF80FF80, 40, 100);
	GC_Debug_RStringT ("It should not be sold, or bundled with", 0xFF80FF80, 40, 120);
	GC_Debug_RStringT ("illegal software or rom images. Also, if", 0xFF80FF80, 40, 140);
	GC_Debug_RStringT ("you suffer from health problems because", 0xFF80FF80, 40, 160);
	GC_Debug_RStringT ("of playing games in this emulator, it's", 0xFF80FF80, 40, 180);
	GC_Debug_RStringT ("your fault because you dont know how to", 0xFF80FF80, 40, 200);
	GC_Debug_RStringT ("read the health and saftey warnings in your", 0xFF80FF80, 40, 220);
	GC_Debug_RStringT ("Gamecube's instruction manual. To agree", 0xFF80FF80, 40, 240);
	GC_Debug_RStringT ("that you are using this emulator fairly", 0xFF80FF80, 40, 260);
	GC_Debug_RStringT ("hold down the L and R triggers on your", 0xFF80FF80, 40, 280);
	GC_Debug_RStringT ("controller. Thank you.", 0xFF80FF80, 40, 300);

	GC_Debug_RStringT ("There is no menu in this version.", 0xFF80FF80, 40, 340);
	GC_Debug_RStringT ("The game will begin after this screen.", 0xFF80FF80, 40, 360);
	GC_Debug_RStringT ("See the readme that was included with", 0xFF80FF80, 40, 380);
	GC_Debug_RStringT ("this emulator for more information.", 0xFF80FF80, 40, 400);
	GC_PAD_Read (&gc_Pad, GC_PAD1); //Read the pad
	while(!((gc_Pad.button & PAD_L) && (gc_Pad.button & PAD_R))){
		GC_PAD_Read (&gc_Pad, GC_PAD1); //Read the pad
	}

	/**** Setup ****/
	cart.rom = (u8*)cart_file;
	cart.pages = (0x80000 / 0x4000);
    cart.crc = 0x43125432;//crc32(0L, cart.rom, size);
	cart.mapper     = MAPPER_SEGA;
    sms.display     = DISPLAY_NTSC;
    sms.territory   = TERRITORY_EXPORT;
	sms.console = CONSOLE_SMS;
	system_assign_device(PORT_A, DEVICE_PAD2B);
    system_assign_device(PORT_B, DEVICE_PAD2B);
	/*** End of Setup ****/

	SetPalette(); //Setup color pallete
	int running = 1;

	GC_Memory_memset((char*)&bitmap, 0, sizeof(bitmap_t));
    bitmap.width  = 640;
    bitmap.height = 480;
    bitmap.depth  = 16;
    bitmap.granularity = 2; /* Bytes per pixel */
    bitmap.pitch = (bitmap.width * bitmap.granularity);
    bitmap.data   = (unsigned char *)genvideo; /* Pointer to your bitmap */
    bitmap.viewport.w = 256; /* Initial size of display on power-up (do not change) */
    bitmap.viewport.h = 192;
    bitmap.viewport.x = 0x20; /* Offset used for rendering (do not change) */
    bitmap.viewport.y = 0;
    //bitmap.remap = 1;

    system_init();	
	GC_AI_SetFreq32KHz();
	snd.sample_rate    = 32000;
    snd.fps            = 60;
    snd.psg_clock      = 3579545;
    snd.fm_clock       = 3579545;
    snd.fm_which       = SND_EMU2413;
    snd.mixer_callback = NULL;
	sound_init();
	system_poweron();
    //system_reset();

	ingame = 1;
	//Setup frameskip
	GC_Video_WaitForVBlank();
	//lasttime = GC_Time_GetTime(); //This will be the time synced to VBlank

	while(running)
	{
		//running = update_input();

		if(!paused)
		{
			/*framecheck++;
			if(framecheck==15){
				lasttime -= 10; //This makes it wait 10ms longer to update next frame
				framecheck=0;
			}*/
            update_input();
			

			/*if((GC_Time_GetTime() - lasttime) > TIME_FRAME){
				//It's behind...
				offby = GC_Time_GetTime() - lasttime - TIME_FRAME;
				lasttime = GC_Time_GetTime() - offby;
				system_frame(1); //Skip frame
			}
			else{*/
				//while((GC_Time_GetTime() - lasttime) < TIME_FRAME);
				//Dont do anything till it's time to update video
				//lasttime = GC_Time_GetTime();

				system_frame(0);

				//Video was resized, black out the background
				if ((lastw != bitmap.viewport.w) || (lasth != bitmap.viewport.h)) {
					GC_Video_ClearFrameBuffer(FrameBuffer, 0x00800080);
					lastw = bitmap.viewport.w;
					lasth = bitmap.viewport.h;
				}
				 
				cenx = (320 - 256)>>1; 
				ceny = (240 - 192)>>1;
				
				vidw = 256;
				vidh = 192;
				for (vidy = 0; vidy < vidh; vidy++) {
					pos = ((320*((vidy+ceny)<<1))+cenx);
					pos2 = (0+(vidy*640));
					for (vidx = 0; vidx < vidw; vidx++) { 
						tempcolor = palette[genvideo[pos2]];
						FrameBuffer[pos] = tempcolor;
						FrameBuffer[pos+320] = tempcolor;
						pos++;
						pos2++;
					}
				}
				GC_Video_WaitForVBlank();

		//	}

			mpointer = (u16*)0x81710000;
			//Merge left and right sound channels into a stereo sample
			for(vidx=0; vidx<snd.buffer_size;vidx++){
				 *mpointer++ = snd.output[0][vidx];
				 *mpointer++ = snd.output[1][vidx];
			}

			GC_Memory_DCFlushRange ((void*)0x81710000, snd.buffer_size*4);
			GC_AI_StopSample();
			GC_AI_LoadSample((u32*)0x81710000,snd.buffer_size*4);
			GC_AI_StartSample();
			if(!audiostarted){
				audiostarted = 1;
			}

        }

		if(paused){
			GC_AI_StopSample(); //Stop Audio
			audiostarted = 0;
			//menu();
			paused = 0;
			framecheck = 0;
			lasttime = GC_Time_GetTime();
		}

		//End of While loop
    }

    system_shutdown();

    return 0;
}

void SetPalette() {
	int i;
	unsigned char r, g, b;

	for (i = 0; i < PALSIZE; i++) {
		r = ((i >> 7) & 0xF8);
		g = ((i >> 2) & 0xF8);
		b = ((i << 3) & 0xF8);
		palette[i] = GC_Video_RGBToYCbCr(r, g, b);
	}
}

void update_input(){
	u8 padno = 0;
	input.system    = 0;
    input.pad[0]    = 0;
    input.pad[1]    = 0;
    input.analog[0] = 0x7F;
    input.analog[1] = 0x7F;

	GC_PAD_Read (&gc_Pad, padno);
	if((gc_Pad.button & PAD_UP) || (gc_Pad.y < -30))
		input.pad[padno] |= INPUT_UP;
	else if((gc_Pad.button & PAD_DOWN) || (gc_Pad.y > 30))
		input.pad[padno] |= INPUT_DOWN;
	if((gc_Pad.button & PAD_LEFT) || (gc_Pad.x < -30))
		input.pad[padno] |= INPUT_LEFT;
	else if((gc_Pad.button & PAD_RIGHT) || (gc_Pad.x > 30))
		input.pad[padno] |= INPUT_RIGHT;
	if(gc_Pad.button & PAD_A)	input.pad[padno] |= INPUT_BUTTON2;
	if(gc_Pad.button & PAD_B)	input.pad[padno] |= INPUT_BUTTON1;
	//if(gc_Pad.button & controller[padno].btnC)	input.pad[padno] |= INPUT_C;
	if(gc_Pad.button & PAD_START)	input.system |= (IS_GG) ? INPUT_START : INPUT_PAUSE;
	//if(gc_Pad.button & controller[padno].btnX)	input.pad[padno] |= INPUT_X;
	//if(gc_Pad.button & controller[padno].btnY)	input.pad[padno] |= INPUT_Y;
	//if(gc_Pad.button & controller[padno].btnZ)	input.pad[padno] |= INPUT_Z;
	//if(gc_Pad.button & controller[padno].btnMode)	input.pad[padno] |= INPUT_MODE;

	if(gc_Pad.button & PAD_L){
		if(gc_Pad.button & PAD_Z){
			asm ("lis 3, 0x8000");
			asm ("ori 3, 3, 0x1800");
			asm ("mtlr 3");
			asm ("blr");
		}
	}
}


